@@ -1,12 +1,14 @@
+#!/usr/bin/env python3
+
 # Python module to feed Asymptote with commands
 # (modified from gnuplot.py)
 from subprocess import *
 class asy:
 	def __init__(self):
-		self.session = Popen(['asy','-q'],stdin=PIPE)
+		self.session = Popen(['asy','-quiet','-inpipe=0','-outpipe=2'],stdin=PIPE)
 		self.help()
 	def send(self, cmd):
-		self.session.stdin.write(cmd+'\n')
+		self.session.stdin.write(bytes(cmd+'\n','utf-8'))
 		self.session.stdin.flush()
 	def size(self, size):
 		self.send("size(%d);" % size)
@@ -23,26 +25,22 @@ class asy:
 	def erase(self):
 		self.send("erase();")
 	def help(self):
-		print "Asymptote session is open.  Available methods are:"
-		print "    help(), size(int), draw(str), fill(str), clip(str), label(str), shipout(str), send(str), erase()"
+		print("Asymptote session is open.  Available methods are:")
+		print("    help(), size(int), draw(str), fill(str), clip(str), label(str), shipout(str), send(str), erase()")
 	def __del__(self):
-		print "closing Asymptote session..."
+		print("closing Asymptote session...")
 		self.send('quit');
 		self.session.stdin.close();
 		self.session.wait()
-		
-
-
-
 
 if __name__=="__main__":
-	g = asy()
+	g=asy()
 	g.size(200)
-	g.draw("unitcircle")
-	g.send("draw(unitsquare)")
-	g.fill("unitsquare, blue")
-	g.clip("unitcircle")
-	g.label("\"$O$\", (0,0), SW")
-	raw_input("press ENTER to continue")
+	g.draw('unitcircle')
+	g.send('draw(unitsquare)')
+	g.fill('unitsquare,blue')
+	g.clip('unitcircle')
+	g.label('"$O$",(0,0),SW')
+	input('press ENTER to continue')
 	g.erase()
 	del g
