@@ -9,6 +9,7 @@
 #define BBOX_H
 
 #include "pair.h"
+#include "settings.h"
 
 namespace camp {
 
@@ -43,17 +44,11 @@ struct bbox {
   {
   }
 
-  bbox(pair p, pair q) 
-  {
-    bbox();
-    add(p); add(q);
-  }
-  
   // Start a bbox with a point
-  bbox(pair z)
+  bbox(const pair& z)
+    : empty(false), left(z.getx()), bottom(z.gety()),
+      right(z.getx()), top(z.gety())
   {
-    bbox();
-    add(z);
   }
 
   bool nonempty() const {
@@ -72,21 +67,59 @@ struct bbox {
     }
     else {
       if (x < left)
-	left = x;  
-      if (x > right)
-	right = x;  
+        left = x;  
+      else if (x > right)
+        right = x;  
       if (y < bottom)
-	bottom = y;
-      if (y > top)
-	top = y;
+        bottom = y;
+      else if (y > top)
+        top = y;
     }
 
     return *this;
   }
 
+  // Add a point to a nonempty bbox
+  void addnonempty(const pair& z)
+  {
+    double x = z.getx(), y = z.gety();
+    if (x < left)
+      left = x;  
+    else if (x > right)
+      right = x;  
+    if (y < bottom)
+      bottom = y;
+    else if (y > top)
+      top = y;
+  }
+
+  // Add a point to a nonempty bbox, updating bounding times
+  void addnonempty(const pair& z, bbox& times, double t)
+  {
+    double x = z.getx(), y = z.gety();
+
+    if (x < left) {
+      left = x;  
+      times.left = t;
+    }
+    else if (x > right) {
+      right = x;  
+      times.right = t;
+    }
+    if (y < bottom) {
+      bottom = y;
+      times.bottom = t;
+    }
+    else if (y > top) {
+      top = y;
+      times.top = t;
+    }
+  }
+
   bbox operator+= (const pair& z)
   {
-    return add(z);
+    add(z);
+    return *this;
   }
 
   bbox operator*= (double x)
@@ -107,13 +140,13 @@ struct bbox {
       return b1;
     else
       return bbox(min(b1.left, b2.left),
-	          max(b1.right, b2.right),
-		  min(b1.bottom, b2.bottom),
-		  max(b1.top, b2.top));
+                  max(b1.right, b2.right),
+                  min(b1.bottom, b2.bottom),
+                  max(b1.top, b2.top));
   }
 
   // Add one bounding box to another
-  bbox add(const bbox& b)
+  void add(const bbox& b)
   {
     if (this->empty)
       *this = b;
@@ -123,13 +156,12 @@ struct bbox {
       bottom = min(bottom, b.bottom);
       top = max(top, b.top);
     }
-
-    return *this;
   }
 
   bbox operator+= (const bbox& b)
   {
-    return add(b);
+    add(b);
+    return *this;
   }
 
   void clip(const bbox& b) {
@@ -191,6 +223,20 @@ inline bbox pad(bbox b1, bbox b2)
   }
 }
 
+inline bbox svgbbox(const bbox& B, pair shift=pair(0,0))
+{
+  bbox b=B;
+  double height=b.top-b.bottom;
+  double threshold=12.0*settings::tex2ps;
+  if(height < threshold) {
+    double offset=threshold-height;
+    b.top += offset;
+    b.bottom += offset;
+  }
+  b.shift(pair(1.99*settings::cm,1.9*settings::cm)+shift);
+  return b;
+}
+
 } // namespace camp
 
 GC_DECLARE_PTRFREE(camp::bbox);
