@@ -18,8 +18,8 @@ protected:
   transform t;
   bool antialias;
 public:
-  drawImage(const transform& t, bool antialias)
-    : t(t), antialias(antialias) {}
+  drawImage(const transform& t, bool antialias, const string& key="")
+    : drawElement(key), t(t), antialias(antialias) {}
   
   virtual ~drawImage() {}
 
@@ -37,8 +37,8 @@ class drawPaletteImage : public drawImage {
   vm::array palette;
 public:
   drawPaletteImage(const vm::array& image, const vm::array& palette,
-                   const transform& t, bool antialias)
-    : drawImage(t,antialias), image(image), palette(palette) {}
+                   const transform& t, bool antialias, const string& key="")
+    : drawImage(t,antialias,key), image(image), palette(palette) {}
   
   virtual ~drawPaletteImage() {}
 
@@ -53,15 +53,16 @@ public:
   }
 
   drawElement *transformed(const transform& T) {
-    return new drawPaletteImage(image,palette,T*t,antialias);
+    return new drawPaletteImage(image,palette,T*t,antialias,KEY);
   }
 };
 
 class drawNoPaletteImage : public drawImage {
   vm::array image;
 public:
-  drawNoPaletteImage(const vm::array& image, const transform& t, bool antialias)
-    : drawImage(t,antialias), image(image) {}
+  drawNoPaletteImage(const vm::array& image, const transform& t,
+                     bool antialias, const string& key="")
+    : drawImage(t,antialias,key), image(image) {}
   
   virtual ~drawNoPaletteImage() {}
 
@@ -74,7 +75,7 @@ public:
   }
 
   drawElement *transformed(const transform& T) {
-    return new drawNoPaletteImage(image,T*t,antialias);
+    return new drawNoPaletteImage(image,T*t,antialias,KEY);
   }
 };
 
@@ -84,8 +85,8 @@ class drawFunctionImage : public drawImage {
   Int width, height;
 public:
   drawFunctionImage(vm::stack *Stack, vm::callable *f, Int width, Int height,
-                     const transform& t, bool antialias)
-    : drawImage(t,antialias), Stack(Stack), f(f),
+                    const transform& t, bool antialias, const string& key="")
+    : drawImage(t,antialias,key), Stack(Stack), f(f),
       width(width), height(height) {}
   
   virtual ~drawFunctionImage() {}
@@ -99,7 +100,7 @@ public:
   }
 
   drawElement *transformed(const transform& T) {
-    return new drawFunctionImage(Stack,f,width,height,T*t,antialias);
+    return new drawFunctionImage(Stack,f,width,height,T*t,antialias,KEY);
   }
 };
 
@@ -108,8 +109,8 @@ class drawRawImage : public drawImage {
   size_t width,height;
 public:
   drawRawImage(unsigned char *raw, size_t width, size_t height,
-               const transform& t, bool antialias)
-    : drawImage(t,antialias), raw(raw), width(width), height(height) {}
+               const transform& t, bool antialias, const string& key="")
+    : drawImage(t,antialias,key), raw(raw), width(width), height(height) {}
   
   virtual ~drawRawImage() {}
 
