@@ -7,16 +7,16 @@
 #ifndef INTERACT_H
 #define INTERACT_H
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "common.h"
 
+void interruptHandler(int);
+
 namespace interact {
 
-extern int interactive;
+extern bool interactive;
 extern bool uptodate;
+extern int lines; // Interactive scroll count
+extern bool query; // Enable interactive scrolling;
 
 void init_interactive();
   
@@ -39,7 +39,7 @@ void cleanup_interactive();
 // This class is used to set a text completion function for readline.  A class
 // is used instead the usual function pointer so that information such as the
 // current environment can be coded into the function (mimicking a closure).
-class completer : public gc {
+class completer {
 public:
   virtual ~completer() {};
   virtual char *operator () (const char *text, int state) = 0;
