@@ -178,11 +178,7 @@ inline void Negate<Int>(vm::stack *s)
 
 inline double pow(double x, double y)
 {
-#ifndef HAVE_POW
-  return exp(y*log(x));
-#else
   return ::pow(x,y);
-#endif
 }
 
 template<class T>
@@ -246,13 +242,15 @@ struct mod {
   }
 };
 
-template <typename T>
+template <typename>
 struct quotient {
-  T operator() (T x, T y,  size_t i=0) {
+  Int operator() (Int x, Int y,  size_t i=0) {
     if(y == 0) dividebyzero(i);
     if(y == -1) return Negate(x);
 // Implementation-independent definition of integer division: round down
-    return (x-portableMod(x,y))/y;
+    Int q=x/y;
+    if(q >= 0 || y*q == x) return q;
+    return q-1;
   }
 };
 
