@@ -72,6 +72,7 @@ public:
   // Returns the possible variable types.  Unlike exp, returns 0 if none
   // match.
   virtual types::ty *varGetType(coenv &e) = 0;
+  virtual trans::varEntry *getCallee(coenv &e, types::signature *sig) = 0;
 
   // As a type:
   // Determines the type, as used in a variable declaration.
@@ -86,8 +87,7 @@ public:
     out << "<base name>";
   }
 
-  virtual symbol *getName() = 0;
-  virtual void *getMarker(coenv &) { return 0; }
+  virtual symbol getName() = 0;
 };
 
 inline ostream& operator<< (ostream& out, const name& n) {
@@ -96,10 +96,10 @@ inline ostream& operator<< (ostream& out, const name& n) {
 }
 
 class simpleName : public name {
-  symbol *id;
+  symbol id;
 
 public:
-  simpleName(position pos, symbol *id)
+  simpleName(position pos, symbol id)
     : name(pos), id(id) {}
 
   trans::varEntry *getVarEntry(coenv &e);
@@ -107,6 +107,7 @@ public:
   // As a variable:
   void varTrans(action act, coenv &e, types::ty *target);
   types::ty *varGetType(coenv &);
+  trans::varEntry *getCallee(coenv &e, types::signature *sig);
 
   // As a type:
   types::ty *typeTrans(coenv &e, bool tacit = false);
@@ -115,18 +116,17 @@ public:
 
   void prettyprint(ostream &out, Int indent);
   void print(ostream& out) const {
-    out << *id;
+    out << id;
   }
-  symbol *getName() {
+  symbol getName() {
     return id;
   }
-  void *getMarker(coenv &e);
 };
 
 
 class qualifiedName : public name {
   name *qualifier;
-  symbol *id;
+  symbol id;
 
   // Gets the record type associated with the qualifier. Reports an
   // error and returns null if the type is not a record.
@@ -141,7 +141,7 @@ class qualifiedName : public name {
   void varTransField(action act, coenv &e,
                      types::ty *target, record *r);
 public:
-  qualifiedName(position pos, name *qualifier, symbol *id)
+  qualifiedName(position pos, name *qualifier, symbol id)
     : name(pos), qualifier(qualifier), id(id) {}
 
   trans::varEntry *getVarEntry(coenv &e);
@@ -149,6 +149,7 @@ public:
   // As a variable:
   void varTrans(action act, coenv &, types::ty *target);
   types::ty *varGetType(coenv &);
+  trans::varEntry *getCallee(coenv &e, types::signature *sig);
 
   // As a type:
   types::ty *typeTrans(coenv &e, bool tacit = false);
@@ -157,9 +158,9 @@ public:
 
   void prettyprint(ostream &out, Int indent);
   void print(ostream& out) const {
-    out << *qualifier << "." << *id;
+    out << *qualifier << "." << id;
   }
-  symbol *getName() {
+  symbol getName() {
     return id;
   }
 };
