@@ -36,11 +36,14 @@ void texdocumentclass(T& out, bool pipe=false)
   
 template<class T>
 void texuserpreamble(T& out,
-                     mem::list<string>& preamble=processData().TeXpreamble)
+                     mem::list<string>& preamble=processData().TeXpreamble,
+                     bool pipe=false)
 {
-  for(mem::list<string>::iterator p=preamble.begin();
-      p != preamble.end(); ++p)
+  for(mem::list<string>::iterator p=preamble.begin(); p != preamble.end();
+      ++p) {
     out << stripblanklines(*p);
+    if(pipe) out << newl << newl;
+  }
 }
   
 template<class T>
@@ -56,19 +59,20 @@ void latexfontencoding(T& out)
 
 template<class T>
 void texpreamble(T& out, mem::list<string>& preamble=processData().TeXpreamble,
-                 bool ASYalign=true)
+                 bool pipe=false, bool ASYbox=true)
 {
-  texuserpreamble(out,preamble);
+  texuserpreamble(out,preamble,pipe);
   string texengine=settings::getSetting<string>("tex");
   if(settings::context(texengine))
     out << "\\disabledirectives[system.errorcontext]%" << newl;
+  if(ASYbox)
+    out << "\\newbox\\ASYbox" << newl
+        << "\\newdimen\\ASYdimen" << newl;
   out << "\\def\\ASYprefix{" << stripFile(settings::outname()) << "}" << newl
-      << "\\newbox\\ASYbox" << newl
-      << "\\newdimen\\ASYdimen" << newl
       << "\\long\\def\\ASYbase#1#2{\\leavevmode\\setbox\\ASYbox=\\hbox{#1}%"
       << "\\ASYdimen=\\ht\\ASYbox%" << newl
       << "\\setbox\\ASYbox=\\hbox{#2}\\lower\\ASYdimen\\box\\ASYbox}" << newl;
-  if(ASYalign)
+  if(!pipe)
     out << "\\long\\def\\ASYaligned(#1,#2)(#3,#4)#5#6#7{\\leavevmode%" << newl
         << "\\setbox\\ASYbox=\\hbox{#7}%" << newl
         << "\\setbox\\ASYbox\\hbox{\\ASYdimen=\\ht\\ASYbox%" << newl
@@ -120,7 +124,7 @@ void texdefines(T& out, mem::list<string>& preamble=processData().TeXpreamble,
                 bool pipe=false)
 {
   if(pipe || !settings::getSetting<bool>("inlinetex"))
-    texpreamble(out,preamble,!pipe);
+    texpreamble(out,preamble,pipe);
 
   if(pipe) {
     // Make tex pipe aware of a previously generated aux file.
@@ -172,8 +176,8 @@ template<class T>
 bool setlatexfont(T& out, const pen& p, const pen& lastpen)
 {
   if(p.size() != lastpen.size() || p.Lineskip() != lastpen.Lineskip()) {
-    out <<  "\\fontsize{" << p.size()*ps2tex << "}{" << p.Lineskip()*ps2tex
-        << "}\\selectfont\n";
+    out <<  "\\fontsize{" << p.size()*settings::ps2tex << "}{" 
+        << p.Lineskip()*settings::ps2tex << "}\\selectfont%" << newl;
     return true;
   }
   return false;
@@ -277,7 +281,7 @@ public:
   void dot(path p, pen, bool newPath=true);
   
   void writeshifted(pair z) {
-    write(conj(z)*ps2tex);
+    write(conj(z)*settings::ps2tex);
   }
   
   void translate(pair z) {}
